// Includings
#include <ctype.h>
#include <util/delay.h>
#include <avr/io.h>
#include <avr/wdt.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>

// Definitionen
#define F_CPU 8000000UL     					//Quarz mit 8.000 Mhz
#define DMX_CHANNELS    (4)						//use 4ch
#define F_OSC			(8000)		  			//oscillator freq. in kHz (typical 8MHz or 16MHz)

// Compilerschalter fr verschiedene Firmwarefunktionen
///////////////////////////////////////////////////////
//#define UseHardcodedStartaddress				//Einbinden, um feste Startadresse zu nutzen
//#define HardcodedStartaddress 1				//Einbinden, um fest einprogrammierte DMX-Startadresse zu definieren
///////////////////////////////////////////////////////

// PWM Register definieren
#define pwm1 OCR0A
#define pwm2 OCR0B
#define pwm3 OCR1AL
#define pwm4 OCR1BL

// Variablendefinitionen
volatile uint8_t	DmxField[DMX_CHANNELS];		//array of DMX vals (raw)
uint8_t				DmxOldField[DMX_CHANNELS];	//array of DMX vals (to check if new data was received)
volatile uint16_t	DmxAddress;					//start address
volatile uint8_t	DmxAlive;					//Zhler fr DMX-Lost-Erkennung

#ifndef UseHardcodedStartaddress
	// Um Flash zu sparen, werden einzelne Bits des Bytes fr die Einstellung verwendet
	volatile uint8_t	DmxAddressSetup;		//Bit 0: GetNewDmxAddress, Bit 1: Catched511, Bit 2: Catched512, Bit 3: NewAddressAbove255
	#define		GNDA	0	// GetNewDMXAddress
	#define		C511	1	// Catched511
	#define		C512	2	// Catched512
	#define		NAA255	4	// NewAddressAbove255
#endif

#define TRUE 1
#define FALSE 0

// Funktionsprototypen
extern void init_DMX(void);
extern void init_PWM(void);
extern void update_PWM(void);

// GPIO Konfigurieren
#define AUXOUT				PD1
#define TASTER				PD3
#define	LEDGREEN			PD4
#define LEDYELLOW			PD6

// GPIO Funktionen definieren
#define ledYellowOn()    	PORTD |= (1<<LEDYELLOW)
#define ledYellowOff()   	PORTD &=~(1<<LEDYELLOW)
#define ledYellowToggle()	PORTD ^= (1<<LEDYELLOW)
#define ledGreenOn()  		PORTD |= (1<<LEDGREEN)
#define ledGreenOff() 		PORTD &=~(1<<LEDGREEN)
#define ledGreenToggle()	PORTD ^= (1<<LEDGREEN)
#define ledBothOn()			PORTD |= (1<<LEDYELLOW)|(1<<LEDGREEN)
#define ledBothOff()		PORTD &=~(1<<LEDYELLOW)|~(1<<LEDGREEN)
#define ledBothToggle()		PORTD ^= (1<<LEDYELLOW)|(1<<LEDGREEN)
